
export { UnicodeCldr }



var _is_in_browser = (typeof __filename === 'undefined' || typeof __dirname === 'undefined')
console.log('--- init unicode cldr in ---', _is_in_browser ? 'browser' : 'Node.js', '---')

function usecldr(info) {
    console.log('--- use unicode cldr ---', info)
}

usecldr('first')

var CldrType = new Object
CldrType.CURRENCY = 'currency'
CldrType.DATE = 'date'
CldrType.NUMBER = 'number'
CldrType.PLURAL = 'plural'

function UnicodeCldr(cldr_version, cldr_from) {
    this.CldrType = CldrType

    console.log('--- cldr version ---', cldr_version)
    this._version = cldr_version
    this._from = cldr_from
    this._rules = {}

    this._loading = 0
    this._onReady = null
    this._preparing = 0
    this._onPrepare = null

    this._preparing += 2

    var cldrjs = './' + this._version + '/cldr_impl.mjs'
    console.log('--- load cldr impl ---', cldrjs)

    import(cldrjs).then((namespaceObject) => {
        this._impl_cldr = namespaceObject.UnicodeCldrImpl
        this._impl = new this._impl_cldr
        console.log('--- impl cldr ---', namespaceObject, this._impl)
        this.checkPrepare()
    }).catch(error => {
        console.log('--- error ---', error)
        this.checkPrepare()
    })

    var localejs = './' + this._version + '/locale_impl.mjs'
    console.log('--- load locale impl ---', localejs)

    import(localejs).then((namespaceObject) => {
        this._impl_locale = namespaceObject.LocaleImpl
        console.log('--- impl locale ---', namespaceObject, this._impl_locale)
        this.checkPrepare()
    }).catch(error => {
        console.log('--- error ---', error)
        this.checkPrepare()
    })

    this._locales = {}
}

UnicodeCldr.prototype.setTypes = function(types) {
    this._types = [].concat(types)

    this.load_common()
}

UnicodeCldr.prototype.checkPrepare = function() {
    this._preparing --
    console.log('--- preparing count ---', this._preparing)
    if (this._preparing == 0 && this._onPrepare) {
        this._onPrepare()
    }
}

UnicodeCldr.prototype.checkReady = function() {
    this._loading --
    console.log('--- loading count ---', this._loading)
    if (this._loading == 0 && this._onReady) {
        this._onReady()
    }
}

UnicodeCldr.prototype.load_rules = function(base, rules, locale) {
    var links = [].concat(base)
    for(var i=0; i<this._types.length; i++) {
        links = links.concat(rules[this._types[i]])
    }

    this._loading += links.length
    for(var i=0; i<links.length; i++) {
        if (links[i] in this._rules) {
            console.log('--- find rule ---')
            this.checkReady()
            continue
        }
        var one = links[i].replace(/\$v\$/g, this._version).replace(/\$l\$/g, locale)
        var uri = this._from + '/' + this._version + '/' + one + '.json'
        uri = uri.replace(/\/\//g, '/')
        import(uri, { assert: { type: 'json' } }).then((namespaceObject) => {
            this._rules[one] = namespaceObject.default
            console.log('--- locale rule ---', one, this._rules[one])
            this.checkReady()
        }).catch(error => {
            console.log('--- error ---', error)
            this.checkReady()
        })
    }
}

UnicodeCldr.prototype.load_common = function(types) {
    var rules = {
        'currency': [
        ]
    }

    this.load_rules([], rules, '')
}

UnicodeCldr.prototype.load_locale = function(locale) {
    var rules = {
        'currency': [
            'cldr-numbers-full-$v$/main/$l$/currencies'
        ]
    }

    this.load_rules([], rules, locale)
}

UnicodeCldr.prototype.version = function() {
    return this._impl.version()
}

UnicodeCldr.prototype.locale = function( locale ) {
    if (locale in this._locales) {
        return this._locales[locale]
    }

    var locale_obj = new this._impl_locale(locale)
    this._locales[locale] = locale_obj
    this.load_locale(locale)
    return locale_obj
}

UnicodeCldr.prototype.getRule = function( item, locale ) {
    var one = item.replace(/\$v\$/g, this._version).replace(/\$l\$/g, locale)
    return this._rules[one]
}

UnicodeCldr.prototype.format = function(method, data, variant) {
    var value = '<>'
    if (method[0] == CldrType.CURRENCY) {
        value = data[0]
        var locale = variant[0]
        var region = variant[1]

        var rule = this.getRule('cldr-numbers-full-$v$/main/$l$/currencies', locale)
        var detail = rule['main'][locale]['numbers']['currencies'][region]
        value = detail['symbol'] + data[0]
        console.log('--- get rule ---', detail, value)
    }

    return value
}

UnicodeCldr.prototype.setOnReady = function(onReady) {
    this._onReady = onReady
}

UnicodeCldr.prototype.setOnPrepare = function(onPrepare) {
    this._onPrepare = onPrepare
}
